#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const commander_1 = __importDefault(require("commander"));
const chalk_1 = __importDefault(require("chalk"));
const cli_logging_1 = require("./lib/cli-logging");
const generateCode_1 = require("./generateCode");
const packageJsonContent = fs_1.default.readFileSync(`${__dirname}/../package.json`, 'utf-8');
const packageJson = JSON.parse(packageJsonContent);
const { version } = packageJson;
const cmd = new commander_1.default.Command('generate')
    .description('Generates typescript code from the spec.')
    .arguments('<spec-filename-or-url>')
    .option('--template <template>', 'the type to output. client, express, express-di or types. Default "client". "server" is an alias for express-di.')
    .option('--filename <filename>', 'the file to output. Defaults to <service-name><output-type>.generated.ts')
    .option('--service-name <name>', 'the name of the service. Defaults to info.title from the open api document')
    .option('--not-required-type <nullable|optional|both>', 'what type fields should be if they are not in the required list. Defaults to undefined', 'optional')
    .option('--readonly-dtos', 'specifies that fields in DTOs, and array types therein should be prefixed with `readonly`. Defaults to false', false)
    .option('--empty-type <unknown|never|{}>', 'what type should be generated for objects with no properties. Defaults to unknown for all templates expect msw-handlers where the default is never', '')
    .action(async (specFilenameOrUrl, options) => {
    const logger = (0, cli_logging_1.processLogger)();
    if (options.template &&
        ![
            'client',
            'server',
            'express',
            'express-di',
            'types',
            'msw-handlers'
        ].includes(options.template.toLowerCase())) {
        commander_1.default.help({ error: true });
    }
    if (!['nullable', 'optional', 'both'].includes(options.notRequiredType)) {
        commander_1.default.help({ error: true });
    }
    const output = (options.template ? options.template : 'client');
    try {
        await (0, generateCode_1.generateCode)({
            openApiDocumentFilenameOrUrl: specFilenameOrUrl,
            output,
            outputFilename: options.filename,
            serviceName: options.serviceName,
            logger,
            nonRequiredType: options.notRequiredType,
            readonlyDTOs: options.readonlyDtos,
            emptyType: options.emptyType || options.template === 'msw-handlers'
                ? 'never'
                : 'unknown'
        });
        logger.log(chalk_1.default.green('\nCompleted successfully!\n\n'));
        process.exit(0);
    }
    catch (e) {
        logger.error(`\nFAILED. ${e instanceof Error
            ? `${e.message}\n${e.stack ?? ''}`
            : 'Unknown error'}\n\n`);
        process.exit(1);
    }
});
commander_1.default.program.version(version).addCommand(cmd).parse();
