"use strict";
/* eslint-disable @typescript-eslint/no-non-null-assertion */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const yaml_1 = __importDefault(require("yaml"));
const json_pointer_1 = __importDefault(require("json-pointer"));
const cli_logging_1 = require("../../lib/cli-logging");
const AsyncJsonWalker_1 = require("./AsyncJsonWalker");
async function fileExists(filename) {
    try {
        await promises_1.default.access(filename);
        return true;
    }
    catch (err) {
        if (typeof err === 'object' &&
            err &&
            'code' in err &&
            err.code === 'ENOENT') {
            return false;
        }
        else {
            throw err;
        }
    }
}
const isReference = (node) => {
    return (typeof node === 'object' &&
        node !== null &&
        '$ref' in node &&
        typeof node.$ref === 'string');
};
class Reference {
    static getAbsolutePath(refBasePath, referencePath) {
        if (path_1.default.isAbsolute(referencePath)) {
            return referencePath;
        }
        return path_1.default.join(path_1.default.dirname(refBasePath), referencePath);
    }
    constructor(reference, refBasePath) {
        const [referencePath, pointer] = reference.split('#');
        this.filename = referencePath
            ? Reference.getAbsolutePath(refBasePath, referencePath)
            : refBasePath;
        this.pointer = pointer;
        this.$ = [this.filename, pointer].join('#');
    }
    filename;
    pointer;
    $;
}
class Loader {
    logger;
    constructor(logger) {
        this.logger = logger;
    }
    files = new Map();
    async loadFromFile(filename) {
        const log = this.logger?.create(`Loading file ${filename}`);
        if (!(await fileExists(filename))) {
            throw new Error(`File ${filename} does not exist`);
        }
        const extension = path_1.default.extname(filename);
        const data = await promises_1.default.readFile(filename, 'utf-8');
        const doc = (() => {
            if (['.yaml', '.yml'].includes(extension)) {
                return yaml_1.default.parse(data);
            }
            if (extension === '.json') {
                return JSON.parse(data);
            }
            throw new Error(`Unsupported extension ${extension}`);
        })();
        log?.((0, cli_logging_1.success)());
        return doc;
    }
    static async makeReferencesAbsolute(filename, doc) {
        await AsyncJsonWalker_1.AsyncJsonWalker.walk(doc, item => {
            if (isReference(item)) {
                const ref = new Reference(item.$ref, filename);
                item.$ref = ref.$;
            }
        });
    }
    async load(filename) {
        if (!this.files.has(filename)) {
            this.files.set(filename, (async () => {
                const doc = await this.loadFromFile(filename);
                await Loader.makeReferencesAbsolute(filename, doc);
                return doc;
            })());
        }
        return this.files.get(filename);
    }
}
const inferTitle = (node, reference) => {
    if (node.title && typeof node.title === 'string') {
        return node.title;
    }
    if (!reference.pointer) {
        return path_1.default.parse(reference.filename).name;
    }
    return reference.pointer.substring(reference.pointer.lastIndexOf('/') + 1);
};
const componentKeyForPointer = (ptr) => {
    const p = json_pointer_1.default.parse(ptr).reverse();
    if (p.includes('schema')) {
        return 'schemas';
    }
    if (p.includes('parameters')) {
        return 'parameters';
    }
    if (p.includes('responses')) {
        return 'responses';
    }
    return 'schemas';
};
const load = async (source, logger) => {
    const filename = path_1.default.isAbsolute(source)
        ? source
        : path_1.default.join(process.cwd(), source);
    const loader = new Loader(logger);
    const doc = (await loader.load(filename));
    const walked = [];
    const walkFn = async (node, ptr) => {
        if (!isReference(node) || walked.includes(ptr)) {
            return;
        }
        walked.push(ptr);
        const ref = new Reference(node.$ref, filename);
        const refDoc = await loader.load(ref.filename);
        const refNode = json_pointer_1.default.get(refDoc, ref.pointer);
        const title = inferTitle(refNode, ref);
        refNode.title = title;
        if (ref.filename !== filename) {
            const newRef = (() => {
                if ('swagger' in doc) {
                    return new Reference(`#/definitions/${title}`, filename);
                }
                const key = componentKeyForPointer(ptr);
                return new Reference(`#/components/${key}/${title}`, filename);
            })();
            json_pointer_1.default.set(doc, newRef.pointer, refNode);
            node.$ref = newRef.$;
            await AsyncJsonWalker_1.AsyncJsonWalker.walk(doc, walkFn, newRef.pointer);
        }
    };
    await AsyncJsonWalker_1.AsyncJsonWalker.walk(doc, walkFn);
    return doc;
};
exports.load = load;
